/*---------------------------------------------------------------
           FILE: cor.c
      $Revision: 2.0 $
         $State: Exp $
  Last submitted $Date: 93/03/30 18:00:36 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	cormain.c,v $
 * Revision 2.0  93/03/30  18:00:36  jh
 * Actual first release version.
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/cormain.c,v $

  Copyright (c) 1993 Joseph Harrington.  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/cormain.c,v 2.0 93/03/30 18:00:36 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1993 Joseph Harrington.  All Rights reserved.";
#endif

/*---------------------------------------------------------------
cor.c = IRAF imfort program to do corelations on a pairs of
        images.

cor a_im b_im out_im

Takes IRAF files a_im and b_im and makes a least squares
corelation image named out_im.  The image dimensions are one
less than the sum in each direction of the dimensions of the
input images.  Pixel values in this image are calculated as
follows:

The images are offset from one another according to the
coordinates of the pixel whose value is being calculated, their
overlaps are computed, corresponding pixels in the overlap
regions are subtracted, and the mean square of this difference
becomes the pixel value in the output image.

The offset is such that the lower left corner of the output image
corresponds to the upper right corner of b_im overlapping the
lower left corner of a_im by one pixel in each dimension.  The
header values of CRPIXn and CRVALn are set.

External object files or libraries:
corparams.c
memory.c
cor.c
irafio.c
-lm
lib$ = the lib directory under iraf
hlib$ = the unix/hlib directory under iraf
lib$/libimfort.a
lib$/libsys.a
lib$/libvops.a
hlib$/libos.a

Depending on operating system, other externals may be necessary
for IRAF imfort.
---------------------------------------------------------------*/
/* definitions */

#define JJKIMCFNMAX 120

/* inclusions */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <imiraf.h>
#include "corparams.h"
#include "irafio.h"
#include "memory.h"
#include "cor.h"

/* data types */

typedef struct IMAGE Image;

/*---------------------------------------------------------------
Effect: Creates a correlation image from two input images.
	Prints diagnostic on stderr and exits with status 1 on
	error.  See help information.

Requires: 2 images in OIF format.

Modifies: File named in third positional argument.

Implementation notes: IMFORT program specific to IRAF OIF format.
---------------------------------------------------------------*/
int main(int argc, char *argv[]){
    Doub2 *a;
    Doub2 *b;
    Doub2 *out;
    Data  *dat;
    Image  i_out;
    int    outd;

    dat = create_data();
    if (!dat){
        exit(1);
    }

    /* parse command line and data file, allocating space as needed */
    if (!interp_command_line_args(argc, argv, dat)){
        exit(1);
    }

    strncpy(i_out.file, dat->out_im, JJKIMCFNMAX);
    i_out.file[JJKIMCFNMAX] = '\0';

    /* get images */
    a = iraf_read_float_image(dat->a_im);
    if (!a){
	exit(1);
    }
    b = iraf_read_float_image(dat->b_im);
    if (!b){
	exit(1);
    }

    /* compute correlation */
    out = (dat->cor)(a, b, dat->x, dat->y, dat->radius);
    if (!out){
	exit(1);
    }

    /* write out */
    if (!iraf_write_float_image(dat->out_im, out)){
	exit(1);
    }

    /* edit output header */
    outd = imrwopen(i_out);
    if (-1 == outd){
	fprintf(stderr, "%s: image header keywords not added.\n",argv[0]);
	exit(1);
    }

    imakwd(outd, "CRPIX1", "", (double)b->ni);
    imakwd(outd, "CRVAL1", "", 0.);
    imakwd(outd, "CD1_1",  "", 1.);
    imakwd(outd, "CRPIX2", "", (double)b->nj);
    imakwd(outd, "CRVAL2", "", 0.);
    imakwd(outd, "CD2_2",  "", 1.);

    imclose(outd);

    /* clean up */
    free(dat);
    free_doub2(a);
    free_doub2(b);
    free_doub2(out);

    return 0;
}
