/*---------------------------------------------------------------
           FILE: corparams.c
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/14 00:42:39 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	corparams.c,v $
 * Revision 2.1  93/05/14  00:42:39  jh
 * Added iprodsum cor
 * 
 * Revision 2.0  93/03/30  18:01:28  jh
 * Actual first release version.
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/corparams.c,v $

  Copyright (c) 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/corparams.c,v 2.1 93/05/14 00:42:39 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
corparams.c = parameter reader/parser for cor

External object files or libraries:
cor.c
---------------------------------------------------------------*/
/* definitions */

#define Streq(s1, s2) ( strcmp( s1, s2 ) == 0 ) /* These strings the same? */

/* inclusions */

#include <string.h>
#include <stdlib.h>
#include "corparams.h"
#include "cor.h"

/*------------------create_data----------------------------------
Effect: Allocates a Data structure and returns a pointer to it.
        Prints an error message and returns NULL on error.

Requires: nothing.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
Data *
create_data(){
    Data *dat;
    
    dat = (Data *)malloc(sizeof(*dat));
    if(!dat){
	fprintf(stderr, "create_data: unable to allocate %u bytes\n",
		sizeof(*dat));
	return NULL;
    }

    /* defaults */
    dat->radius = DBL_MAX;
    dat->x      = 0.;
    dat->y      = 0.;
    dat->cor    = snscor;

    /* other inits */
    dat->a_im[0]   = '\0';
    dat->b_im[0]   = '\0';
    dat->out_im[0] = '\0';

    return dat;
}

/*---------------------interp_command_line_args------------------
Effect: Parses command line and stuffs dat struct appropriately.
        Returns SUCCESS if parsed correctly and FAILURE if not.

Requires: argc and argv: as from main()
          dat: pointer to dat structure of stuff.

Modifies: *dat.

Implementation notes:
---------------------------------------------------------------*/
int
interp_command_line_args( 
    int          argc,
    char        *argv[],
    Data        *dat
){
    int i; /* argument loop counter */
    int j = 0; /* positional argument counter */
    int minpos = 3; /* minimal number of positional arguments */
    int maxpos = 3; /* maximum number of positional arguments */
    char **posv;

    posv = (char **)malloc(argc * sizeof(*posv));
    if (!posv){
		fprintf(stderr,
		     "interp_command_line_args: unable to allocate %u bytes.\n",
		argc * sizeof(*posv));
	return FAILURE;
    }

    for ( i = 1 ; i < argc ; i++ ) {
	if(   Streq( argv[i], "-radius" )
	   || Streq( argv[i], "-r" ) ) {
	    dat->radius = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-x" ) ) {
	    dat->x = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-y" ) ) {
	    dat->y = atof(argv[++i]);
	}
	else if( Streq( argv[i], "-cor" ) ) {
	    i++;
	    if( Streq( argv[i], "sns") ) {
		dat->cor = snscor;
	    } else if( Streq( argv[i], "snc") ) {
		dat->cor = sncor;
	    } else if( Streq( argv[i], "iprod") ) {
		dat->cor = iprodcor;
	    } else if( Streq( argv[i], "iprodsum") ) {
		dat->cor = iprodsumcor;
	    } else {
		fprintf(stderr, "%s: unknown correlation function %s.\n",
		argv[0], argv[i]);
		usage(argv[0]);
		return FAILURE;
	    }
	}
	else if( Streq( argv[i], "-h" ) ||
		Streq( argv[i], "-help" ) ) {
	    usage(argv[0]);
	    return FAILURE;
  	}
	else { /* it's positional */
	    posv[j++] = argv[i];
	}
    }

    /* do stuff with positional arguments */
    if ( j < minpos || j > maxpos ) {
	fprintf(stderr, 
		"%s: have %d positional argument%s, need %d\n",
		argv[0], j, ( j == 1 ? "" : "s" ), minpos);
	usage(argv[0]);
	return FAILURE;
    }
    i = 0;
    strncpy( dat->a_im, posv[i++], FILENAME_MAX-1);
    strncpy( dat->b_im, posv[i++], FILENAME_MAX-1);
    strncpy( dat->out_im, posv[i++], FILENAME_MAX-1);

    return SUCCESS;
}

/*------------------------usage----------------------------------
Effect: Prints the program usage.

Requires: progname: pointer to character string to print.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
void 
usage(char *progname)
{
    fprintf(stderr, "%s: correlate two IRAF images.\n", progname);
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "%s a_im b_im out_im <options>\n", progname);
    fprintf(stderr, "\ta_im and b_im are the names of the input images\n");
    fprintf(stderr, "\tout_im is the name of the output image\n");
    fprintf(stderr, "\toptions:\n");
    fprintf(stderr,
        "\t-radius (-r) double: maximum shift in pixels from nominal match,\n");
    fprintf(stderr, "\t\t\tdefault any overlap allowed\n");
    fprintf(stderr, "\t-x double: horizontal nominal match (for radius),\n");
    fprintf(stderr, "\t\t\tdefault 0.0\n");
    fprintf(stderr, "\t-y double: vertical nominal match (for radius),\n");
    fprintf(stderr, "\t\t\tdefault 0.0\n");
    fprintf(stderr, "\t-cor function name: correlation function to use,\n");
    fprintf(stderr, "\t\t\tdefault \"sns\"\n");
    fprintf(stderr, "\t\t\toptions: \"sns\"  squared noise-to-signal ratio\n");
    fprintf(stderr, "\t\t\t         \"snc\"   squared noise\n");
    fprintf(stderr,
	     "\t\t\t         \"iprod\" inverse mean product (like crosscor)\n");
    fprintf(stderr,
	 "\t\t\t         \"iprodsum\" inverse total product (like crosscor)\n");
    fprintf(stderr, "\t-help (-h): print this message and quit.\n");

    return;
}
