/*---------------------------------------------------------------
           FILE: irafmosdat.c
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/11 20:03:05 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	irafmosdat.c,v $
 * Revision 2.1  93/05/11  20:03:05  jh
 * Passes intensity shift values through without modifying them.
 * 
 * Revision 2.0  93/03/30  18:02:41  jh
 * Actual first release version.
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/irafmosdat.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/irafmosdat.c,v 2.1 93/05/11 20:03:05 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
irafmosdat.c = functions to read IRAF mosaic data files for
               jiggle program.

External object files or libraries:
memory.c
---------------------------------------------------------------*/
/* definitions */

#define Streq(s1, s2) ( strcmp( s1, s2 ) == 0 ) /* These strings the same? */
#define Min(a, b) ((a) < (b) ? (a) : (b))
#define Max(a, b) ((a) > (b) ? (a) : (b))
#define LINELEN 80

/* inclusions */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include "memory.h"
#include "moseval.h"
#include "jigparams.h"
#include "irafmosdat.h"

/*----------------------iraf_read_mdb----------------------------
Effect: Reads IRAF mosaic database file.  Stuffs *dat
        appropriately.  Prints appropriate messages on error,
	returns SUCCESS on success and FAILURE on error.

Requires: dat: pointer to Data that has already had command line
               args read into it.

Modifies: *dat

Implementation notes:
---------------------------------------------------------------*/
int
iraf_read_mdb(Data *dat){
    size_t  i;
    size_t  j;
    FILE   *file;
    char    filename[FILENAME_MAX];
    size_t  fnlen;
    char    line[LINELEN + 1];
    char   *fgetsret;
    char    scanfstr[LINELEN];
    char    keyword[LINELEN + 1];
    char    value[LINELEN];
    int     scanfret;
    double  xshift;
    double  yshift;
    char   *lp;
    char   *tp;
    char    tmpstr[LINELEN];

    if ( strlen(dat->mosaic) + strlen(dat->dbext) > FILENAME_MAX ){
	fprintf(stderr, 
		"read_mdb: path too long.  Max filename is %u bytes.\n",
		FILENAME_MAX);
	return FAILURE;
    }
    strncpy(filename, dat->mosaic, FILENAME_MAX);
    fnlen = strlen(filename);
    strncpy(&(filename[fnlen]), dat->dbext, FILENAME_MAX - fnlen);

    file = fopen( filename, "r");
    if (!file){
	perror(filename);
	return FAILURE;
    }

    sprintf(scanfstr, "%%%ds %%%ds\n", LINELEN, LINELEN);

    while(1){
	fgetsret = fgets(line, LINELEN, file);
        if ( fgetsret == NULL ){
	    perror(filename);
	    fprintf(stderr, "read_mdb: Mosaic database file corrupt.\n");
	    fclose(file);
	    return FAILURE;
 	}
	scanfret = sscanf(line, scanfstr, keyword, value);

	if ( Streq(keyword, "ncols") && scanfret == 2 ){
	    dat->nx = atoi(value);
	}
	else if ( Streq(keyword, "nrows") && scanfret == 2 ){
	    dat->ny = atoi(value);
	}
	else if ( Streq(keyword, "nxsub") && scanfret == 2 ){
	    dat->nj = atoi(value);
	}
	else if ( Streq(keyword, "nysub") && scanfret == 2 ){
	    dat->ni = atoi(value);
	}
	else if ( Streq(keyword, "nxoverlap") && scanfret == 2 ){
	    dat->mdbxovlap = atoi(value);
	}
	else if ( Streq(keyword, "nyoverlap") && scanfret == 2 ){
	    dat->mdbyovlap = atoi(value);
	}
	else if ( Streq(keyword, "corner") && !Streq(value, "ll")  ||
		  Streq(keyword, "order")  && !Streq(value, "row") ||
		  Streq(keyword, "raster") && !Streq(value, "no")
		 ){
	   fprintf(stderr,
            "read_mdb: Mosaic database and shifts files not in right order.\n");
	   fprintf(stderr, "\tRun 'sortmos %s' in IRAF.\n", dat->mosaic);
	   fclose(file);
	   return FAILURE;
	}
	else if ( Streq(keyword, "nsubrasters") ){
	    break;
	}
    }

    /* malloc 2D arrays */
    dat->pieces = alloc_char3(dat->ni, dat->nj, LINELEN);
    if (!dat->pieces){
	fclose(file);
	return FAILURE;
    }
    dat->zshift = alloc_char3(dat->ni, dat->nj, LINELEN);
    if (!dat->zshift){
	fclose(file);
	return FAILURE;
    }
    
    sprintf(scanfstr, " %%%d[^[ \t]", LINELEN);

    /* read and stuff piece names */
    for ( i = 0 ; i < dat->ni ; i++ ){
	for ( j = 0 ; j < dat->nj ; j++ ){
	    fgetsret = fgets(line, LINELEN, file);
	    if ( fgetsret == NULL ){
		perror(filename);
		fprintf(stderr, "read_mdb: Mosaic database file corrupt.\n");
		fclose(file);
		return FAILURE;
	    }
	    sscanf(line, scanfstr, value);
	    if ( strlen(value) + strlen(dat->imext) > FILENAME_MAX ){
		fprintf(stderr, 
		  "read_mdb: image name too long.  Max filename is %u bytes.\n",
			FILENAME_MAX);
		fclose(file);
		return FAILURE;
	    }
	    strncpy(dat->pieces->data[i][j], value, LINELEN);
	    fnlen = strlen(dat->pieces->data[i][j]);
	    strncpy(&(dat->pieces->data[i][j][fnlen]), dat->imext,
		    FILENAME_MAX - fnlen);
	}
    }
    fclose(file);

    /* read nominal shifts from mosaic shift file */
    if ( strlen(dat->mosaic) + strlen(dat->sfext) > FILENAME_MAX ){
	fprintf(stderr, 
	     "read_mdb: shift file name too long.  Max filename is %u bytes.\n",
		FILENAME_MAX);
	return FAILURE;
    }
    strncpy(filename, dat->mosaic, FILENAME_MAX);
    fnlen = strlen(filename);
    strncpy(&(filename[fnlen]), dat->sfext, FILENAME_MAX - fnlen);

    file = fopen( filename, "r");
    if (!file){
	perror(filename);
	return FAILURE;
    }

    for (     i = 0 ; i < dat->ni ; i++ ){
	for ( j = 0 ; j < dat->nj ; j++ ){
	    fgetsret = fgets(line, LINELEN, file);
	    if ( fgetsret == NULL ){
		if ( ferror(file) ){
		    perror(filename);
		    fprintf(stderr,
			    "read_mdb: Problem with mosaic shift file.\n");
		} else {
		    fprintf(stderr, "read_mdb: Mosaic shift file too short.\n");
		}
		fclose(file);
		return FAILURE;
	    }

	    /* parse the line */
	    lp = line;
	    while (isspace(*lp++)){} /* kill leading blank space */
	    lp--; /* leave us pointing at the first non-whitespace */

	    /* read x and y shifts */
	    tp = tmpstr;
            while (!isspace(*lp) && *lp != '\0'){
                *tp++ = *lp++;
            }
            *tp = '\0';
	    xshift = atof(tmpstr);
	    if (*lp == '\0'){
		fprintf(stderr,
			    "read_mdb: Mosaic shift file line %d too short.\n",
			i * dat->nj + j);
		fclose(file);
		return FAILURE;
	    }

	    while (isspace(*lp++)){} /* kill blank space */
	    lp--; /* leave us pointing at the first non-whitespace */

	    tp = tmpstr;
            while (!isspace(*lp) && *lp != '\0'){
                *tp++ = *lp++;
            }
            *tp = '\0';
	    yshift = atof(tmpstr);

	    /* read intensity shift */
	    strncpy(dat->zshift->data[i][j], lp, LINELEN);

	    /* calculate nominal overlap */
	    if ( i == 0 && j == 0 ){
		dat->llcx = xshift;
		dat->llcy = yshift;
	    } else if ( i == 1 && j == 0 ){
		dat->yovlap = dat->llcy - yshift + dat->mdbyovlap;
	    } else if ( i == 0 && j == 1 ){
		dat->xovlap = dat->llcx - xshift + dat->mdbxovlap;
	    }
	}
    }
    fclose(file);

    return SUCCESS;
}

/*---------------------iraf_write_shifts-------------------------
Effect: Writes shifts for the irmosaic series of IRAF
        tasks.  Returns SUCCESS or FAILURE as appropriate.

Requires: dat: pointer to Data parameter structure.

          simp: pointer to Simplex_param with results of
	        minimization.

Modifies: nothing.

Implementation notes: Commented out replacing existing file; may
                      want to put this in as an option later.
---------------------------------------------------------------*/
int
iraf_write_shifts(Data *dat, Simplex_param *simp){
    size_t  i;
    size_t  j;
#ifdef DELETED
    FILE   *file;
#endif
    char    filename[FILENAME_MAX];
    size_t  fnlen;

#ifdef DELETED
    /* write new mosaic shift file */
    /* create new filename and open it */
    if ( strlen(dat->mosaic) + strlen(dat->sfext) > FILENAME_MAX ){
	fprintf(stderr, 
	     "read_mdb: shift file name too long.  Max filename is %u bytes.\n",
		FILENAME_MAX);
	return FAILURE;
    }
    strncpy(filename, dat->mosaic, FILENAME_MAX);
    fnlen = strlen(filename);
    strncpy(&(filename[fnlen]), dat->sfext, FILENAME_MAX - fnlen);

    file = fopen(filename, "w");
    if (!file){
	perror(filename);
	return FAILURE;
    }
#endif

    /* write shifts */
    for ( i = 0 ; i < dat->ni ; i++ ){
	for ( j = 0 ; j < dat->nj ; j++ ){
	    fprintf(stdout, "%10.2lf\t%10.2lf%s", 
		    dat->llcx - j * (dat->nx - dat->mdbxovlap)
		    + (  Mos_Xoff(i, j, dat->nj) < 0
		       ? 0.
		       : simp->p[0][Mos_Xoff(i, j, dat->nj)] ),
		    dat->llcy - i * (dat->ny - dat->mdbyovlap)
		    + (  Mos_Yoff(i, j, dat->nj) < 0
		       ? 0.
		       : simp->p[0][Mos_Yoff(i, j, dat->nj)] ),
		    dat->zshift->data[i][j] );
	}
    }
#ifdef DELETED
    fclose(file);
#endif

    return SUCCESS;
}
