/*---------------------------------------------------------------
           FILE: jiggle.c
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/11 20:02:28 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	jiggle.c,v $
 * Revision 2.1  93/05/11  20:02:28  jh
 * Passes intensity shift values through without modifying them.
 * 
 * Revision 2.0  93/03/30  18:03:46  jh
 * Actual first release version.
 * 
 * Revision 1.2  93/03/30  13:54:20  jh
 * General code cleanup, testing, and added features.
 * First release version.
 * 
 * Revision 1.1  92/12/15  13:05:09  jh
 * Initial revision
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jiggle.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jiggle.c,v 2.1 93/05/11 20:02:28 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
jiggle.c = program that calculates mosaic alignments

External object files or libraries:

jigparams.c
irafio.c
irafmosdat.c
cor.c
simplex.c
memory.c
moseval.c
bilin_interp.c
-ljjkimc
-ljjkutil
-lm
lib$ = the lib directory under iraf
hlib$ = the unix/hlib directory under iraf
lib$/libimfort.a
lib$/libsys.a
lib$/libvops.a
hlib$/libos.a

Depending on operating system, other externals may be necessary
for IRAF imfort.
---------------------------------------------------------------*/
/* definitions */

#define SIDE         1.0
#define RESTART_SIDE 0.1

/* inclusions */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include "moseval.h"
#include "memory.h"
#include "simplex.h"
#include "jigparams.h"
#include "irafio.h"
#include "irafmosdat.h"

/*---------------------------------------------------------------
Effect: Prints optimized shifts file for a mosaic on stdout.
	Prints diagnostic on stderr and exits with status 1 on
	error.  See help information.

Requires: IRAF irmosaic including database, initial shifts file,
	  and all frames in OIF format.

Modifies: nothing.

Implementation notes: IMFORT program specific to IRAF irmosaic
		      mosaics in OIF format.
---------------------------------------------------------------*/
int
main(int argc, char *argv[]){
    Mosevalpars   *mos;
    Doub2p2       *images;
    Simplex_param *simp;
    Data          *dat;
    size_t         i;
    size_t         j;

    dat = create_data();
    if (!dat){
	exit(1);
    }

    /* parse command line and data file, allocating space as needed */
    if (!interp_command_line_args(argc, argv, dat)){
	exit(1);
    }
    if (!iraf_read_mdb(dat)){
	exit(1);
    }

    /* Initialize Moseval */
    mos = create_mosevalpars(dat->ni, dat->nj);
    if (!mos){
	exit(1);
    }
    mos->interp = dat->interp;
    mos->cor    = dat->cor;
    mos->eval   = dat->eval;

    /* build a structure for the images */
    images = alloc_doub2p2(dat->ni, dat->nj);
    if (!images){
	exit(1);
    }
    if(!alloc_arrdoub2p2elts(images->data, dat->ni, dat->nj, dat->ny, dat->nx)){
	exit(1);
    }

    /* read IRAF images */
    for ( i = 0 ; i < dat->ni ; i++){
	for ( j = 0 ; j < dat->nj ; j++){
	    images->data[i][j] = iraf_read_float_image(dat->pieces->data[i][j]);
	    if (!images->data[i][j]){
		exit(1);
	    }
	}
    }

    /* do correlations */
    if(!moseval_do_cors(mos, images, dat->xovlap, dat->yovlap, dat->radius)){
	exit(1);
    }

    /* Initialize Simplex */
    simp = simplex_create( ( (dat->ni * dat->nj) - 1 ) * 2);

    (void)simplex_init_param(simp, mos->eval, mos, 
			     dat->ftol, dat->flip, dat->extend,
			     dat->contract, dat->funcmax);

    moseval_init_simplex_point(mos, simp,
			       dat->xovlap, dat->yovlap, dat->nx, dat->ny);
    (void)simplex_init_point(simp, SIDE);

    /* Call Minimizer */
    (void)simplex(simp);

    /* restart */
    simp->funcmax *= 2;
    (void)simplex_init_point(simp, RESTART_SIDE);
    (void)simplex(simp);

    /* print table */
    iraf_write_shifts(dat, simp);

    /* print diagnostic info */
    if ( dat->diag ){
	print_diag(dat, mos, simp);
    }

    /* free memory */
    simplex_free(simp);

    free_arrdoub2p2elts(images->data, dat->ni, dat->nj);
    free_doub2p2(images);

    free_mosevalpars(mos);

    free_char3(dat->pieces);
    free_char3(dat->zshift);
    free(dat);

    /* end */

    return 0;
}
