/*---------------------------------------------------------------
           FILE: jigparams.c
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/14 00:41:58 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	jigparams.c,v $
 * Revision 2.1  93/05/14  00:41:58  jh
 * Added iprodsum cor
 * 
 * Revision 2.0  93/03/30  18:03:52  jh
 * Actual first release version.
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jigparams.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jigparams.c,v 2.1 93/05/14 00:41:58 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
jigparams.c = parameter reader/parser for jiggle

External object files or libraries:
cor.c
bilin_interp.c
moseval.c
---------------------------------------------------------------*/
/* definitions */

#define Streq(s1, s2) ( strcmp( s1, s2 ) == 0 ) /* These strings the same? */

/* inclusions */

#include <string.h>
#include <stdlib.h>
#include "jigparams.h"
#include "cor.h"
#include "bilin_interp.h"
#include "moseval.h"

/*------------------create_data----------------------------------
Effect: Allocates a Data structure and returns a pointer to it.
        Prints an error message and returns NULL on error.

Requires: nothing.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
Data *
create_data(){
    Data *dat;
    
    dat = (Data *)malloc(sizeof(*dat));
    if(!dat){
	fprintf(stderr, "create_data: unable to allocate %u bytes\n",
		sizeof(*dat));
	return NULL;
    }

    /* defaults */
    dat->radius   = DBL_MAX;
    dat->ftol     = 1e-7;
    dat->flip     = 1.;
    dat->extend   = 2.;
    dat->contract = 0.5;
    dat->funcmax  = 50000;
    dat->diag     = FALSE;

    strncpy(dat->dbext, ".mdb", FILENAME_MAX-1);
    dat->dbext[FILENAME_MAX-1] = '\0';

    strncpy(dat->sfext, ".pos", FILENAME_MAX-1);
    dat->sfext[FILENAME_MAX-1] = '\0';

    strncpy(dat->imext, ".imh", FILENAME_MAX-1);
    dat->sfext[FILENAME_MAX-1] = '\0';

    /* default functions to use */

    dat->interp = bilin_interp;
    dat->cor    = snscor;
    dat->eval   = moseval;

    /* other inits */
    dat->ni = 0;
    dat->nj = 0;
    dat->ny = 0;
    dat->nx = 0;
    dat->xovlap = 0;
    dat->yovlap = 0;

    return dat;
}

/*---------------------interp_command_line_args------------------
Effect: Parses command line and stuffs dat struct appropriately.
        Returns SUCCESS if parsed correctly and FAILURE if not.

Requires: argc and argv: as from main()
          dat: pointer to dat structure to stuff.

Modifies: *dat.

Implementation notes:
---------------------------------------------------------------*/
int
interp_command_line_args( 
    int          argc,
    char        *argv[],
    Data        *dat
){
    int i; /* argument loop counter */
    int j = 0; /* positional argument counter */
    int minpos = 1; /* minimal number of positional arguments */
    int maxpos = 1; /* maximum number of positional arguments */
    char **posv;

    posv = (char **)malloc(argc * sizeof(*posv));
    if (!posv){
		fprintf(stderr,
		     "interp_command_line_args: unable to allocate %u bytes.\n",
		argc * sizeof(*posv));
	return FAILURE;
    }

    for ( i = 1 ; i < argc ; i++ ) {
	if(   Streq( argv[i], "-radius" )
	   || Streq( argv[i], "-r" ) ) {
	    dat->radius = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-dbext" )
		|| Streq( argv[i], "-dx" ) ) {
	    strncpy( dat->dbext, argv[++i], FILENAME_MAX-1 );
	}
	else if(   Streq( argv[i], "-sfext" )
		|| Streq( argv[i], "-sx" ) ) {
	    strncpy( dat->sfext, argv[++i], FILENAME_MAX-1 );
	}
	else if(   Streq( argv[i], "-imext" )
		|| Streq( argv[i], "-ix" ) ) {
	    strncpy( dat->imext, argv[++i], FILENAME_MAX-1 );
	}
	else if( Streq( argv[i], "-ftol" ) ) {
	    dat->ftol = atof(argv[++i]);
	}
	else if( Streq( argv[i], "-flip" ) ) {
	    dat->flip = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-extend" )
		|| Streq( argv[i], "-ex" ) ) {
	    dat->extend = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-contract" )
		|| Streq( argv[i], "-ct" ) ) {
	    dat->contract = atof(argv[++i]);
	}
	else if(   Streq( argv[i], "-funcmax" )
		|| Streq( argv[i], "-fm" ) ) {
	    dat->funcmax = atoi(argv[++i]);
	}
	else if( Streq( argv[i], "-cor" ) ) {
	    i++;
	    if( Streq( argv[i], "sns") ) {
		dat->cor = snscor;
	    } else if( Streq( argv[i], "snc") ) {
		dat->cor = sncor;
	    } else if( Streq( argv[i], "iprod") ) {
		dat->cor = iprodcor;
	    } else if( Streq( argv[i], "iprodsum") ) {
		dat->cor = iprodsumcor;
	    } else {
		fprintf(stderr, "%s: unknown correlation function %s.\n",
		argv[0], argv[i]);
		usage(argv[0]);
		return FAILURE;
	    }
	}
	else if(   Streq( argv[i], "-interp" )
		|| Streq( argv[i], "-int" ) ) {
	    i++;
	    if( Streq( argv[i], "bilin") ) {
		dat->interp = bilin_interp;
	    } else {
		fprintf(stderr, "%s: unknown interpolation function %s.\n",
		argv[0], argv[i]);
		usage(argv[0]);
		return FAILURE;
	    }
	}
	else if( Streq( argv[i], "-eval" ) ) {
	    i++;
	    if( Streq( argv[i], "moseval") ) {
		dat->eval = moseval;
	    } else {
		fprintf(stderr, "%s: unknown mosaic evaluation function %s.\n",
		argv[0], argv[i]);
		usage(argv[0]);
		return FAILURE;
	    }
	}
	else if( Streq( argv[i], "-diag" ) ) {
	    dat->diag = TRUE;
  	}
	else if( Streq( argv[i], "-h" ) ||
		Streq( argv[i], "-help" ) ) {
	    usage(argv[0]);
	    return FAILURE;
  	}
	else { /* it's positional */
	    posv[j++] = argv[i];
	}
    }

    /* do stuff with positional arguments */
    if ( j < minpos || j > maxpos ) {
	fprintf(stderr, 
		"%s: have %d positional argument%s, need %d\n",
		argv[0], j, ( j == 1 ? "" : "s" ), minpos);
	usage(argv[0]);
	return FAILURE;
    }
    i = 0;
    strncpy( dat->mosaic, posv[i++], FILENAME_MAX-1);

    return SUCCESS;
}

/*------------------------usage----------------------------------
Effect: Prints the program usage.

Requires: progname: pointer to character string to print.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
void 
usage(char *progname)
{
    fprintf(stderr,
	    "%s: align parts of an IRAF irmosaic and print shifts table.\n",
	    progname);
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "%s mosaic <options>\n", progname);
    fprintf(stderr, "\tmosaic is the name of the mosaic to align\n");
    fprintf(stderr, "\toptions:\n");
    fprintf(stderr,
        "\t-radius (-r) double: maximum shift in pixels from nominal match,\n");
    fprintf(stderr, "\t\t\tdefault any overlap allowed\n");
    fprintf(stderr,
	    "\t-dbext (-dx) string: extension for mosaic database filename,\n");
    fprintf(stderr, "\t\t\tdefault \".mdb\"\n");
    fprintf(stderr, 
	       "\t-sfext (-sx) string: extension for mosaic shift filename,\n");
    fprintf(stderr, "\t\t\tdefault \".pos\"\n");
    fprintf(stderr, "\t-imext (-ix) string: extension for image filenames,\n");
    fprintf(stderr, "\t\t\tdefault \".imh\"\n");
    fprintf(stderr,
      "\t-ftol double: fractional tolerence for minimization convergence,\n");
    fprintf(stderr, "\t\t\tdefault 1e-7\n");
    fprintf(stderr,
      "\t-flip double: flip parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault 1\n");
    fprintf(stderr,
     "\t-extend (-ex) double: extend parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault 2\n");
    fprintf(stderr,
 "\t-contract (-ct) double: contract parameter, see Numerical recipes 10.4,\n");
    fprintf(stderr, "\t\t\tdefault .5\n");
    fprintf(stderr,
"\t-funcmax (-fm) unsigned integer: maximum number of function evaluations,\n");
    fprintf(stderr, "\t\t\tdefault 50000\n");
    fprintf(stderr,
      "\t-cor function name: correlation function to use,\n");
    fprintf(stderr, "\t\t\tdefault \"sns\"\n");
    fprintf(stderr,
                  "\t\t\toptions: \"sns\"  squared noise-to-signal ratio\n");
    fprintf(stderr, "\t\t\t         \"snc\"   squared noise\n");
    fprintf(stderr,
	     "\t\t\t         \"iprod\" inverse mean product (like crosscor)\n");
    fprintf(stderr,
	 "\t\t\t         \"iprodsum\" inverse total product (like crosscor)\n");
    fprintf(stderr,
      "\t-interp (-int) function name: 2D interpolation function to use,\n");
    fprintf(stderr, "\t\t\tdefault \"bilin\"\n");
    fprintf(stderr,
	    "\t\t\toption: \"bilin\"  bilinear\n");
    fprintf(stderr,
      "\t-eval function name: mosaic evaluation function to use,\n");
    fprintf(stderr, "\t\t\tdefault \"moseval\"\n");
    fprintf(stderr,
	    "\t\t\toption: \"moseval\"  sum of overlap corellations\n");
    fprintf(stderr, "\t-diag: print diagnostic information when done\n");
    fprintf(stderr, "\t-help (-h): print this message and quit.\n");

    return;
}

/*----------------------print_diag-------------------------------
Effect: prints diagnostic extracted from arguments on stderr.

Requires: dat: pointer to Data.
          mos: pointer to Mosevalpars.
	  simp: pointer to Simplex_param.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
void
print_diag(
	   Data          *dat,
	   Mosevalpars   *mos,
	   Simplex_param *simp
){
    fprintf(stderr, "function evals: %u\n", simp->nfunc);

    return;
}
