/*---------------------------------------------------------------
           FILE: jigparams.h
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/11 20:03:34 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	jigparams.h,v $
 * Revision 2.1  93/05/11  20:03:34  jh
 * Passes intensity shift values through without modifying them.
 * 
 * Revision 2.0  93/03/30  18:06:01  jh
 * Actual first release version.
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jigparams.h,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef JIGPARAMS_PROTECT
#ifndef LINT
static char rcsidjigparamsh[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/jigparams.h,v 2.1 93/05/11 20:03:34 jh Exp Locker: jh $";
static char copyrightjigparamsh[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
#endif

/*---------------------------------------------------------------
jigparams.h = include file for jigparams.c
---------------------------------------------------------------*/
#ifndef JIGPARAMS_PROTECT
#define JIGPARAMS_PROTECT

/* definitions */

#define SUCCESS 1
#define FAILURE 0
#define TRUE 1
#define FALSE 0

/* inclusions */

#include <stdio.h>
#include <stddef.h>
#include "memory.h"
#include "moseval.h"
#include "simplex.h"

/* data types */

typedef struct {
    char         mosaic[FILENAME_MAX]; /* name of mosaic */
    char         dbext[FILENAME_MAX];  /* database filename extension */
    char         sfext[FILENAME_MAX];  /* shift filename extension */
    char         imext[FILENAME_MAX];  /* image filename extension */
    size_t       ni;		       /* number of rows of frames */
    size_t       nj;		       /* number of frames in a row */
    size_t       ny;		       /* number of rows of pixels in a frame */
    size_t       nx;		       /* number of pixels in a row */
    int          mdbxovlap;	       /* overlap in pre-mosaic frame */
    int          mdbyovlap;
    Char3       *pieces;	       /* filenames of the pieces */
    double       llcx;		       /* lower-left corner position */
    double       llcy;
    double       xovlap;	       /* nominal overlaps */
    double       yovlap;
    Char3       *zshift;	       /* intensity shift values, if any */
    double       radius;	       /* max offset from nominal overlaps */
    double       ftol;		       /* parameters for minimizer, see */
    double       flip;		       /* simplex.c and Numerical Recipes 10.4*/
    double       extend;
    double       contract;
    unsigned int funcmax;
    int          diag;                 /* print diagnostic info when done? */

    double  (*interp)(                 /* needs to be here because mos isn't */
		      Doub2  *z,       /* defined when command line is read */
		      double  y,
		      double  x
	    ); /* interpolation function */

    Doub2 * (*cor)(
		   Doub2  *a, 
		   Doub2  *b,
		   double  nomx,
		   double  nomy,
		   double  radius
	    ); /* correlation function */

    double  (*eval)(
		   double                 p[],
		   struct st_mosevalpars *mos
	    ); /* mosaic evaluation function */

} Data;

/* prototypes */

#if defined(__STDC__) || defined(__cplusplus)
# define P_(s) s
#else
# define P_(s) ()
#endif


/* jigparams.c */
Data *create_data P_((void));
int interp_command_line_args P_((int argc, char *argv[], Data *dat));
void usage P_((char *progname));
void print_diag P_((Data *dat, Mosevalpars *mos, Simplex_param *simp));

#undef P_

#endif
