/*---------------------------------------------------------------
           FILE: moseval.c
      $Revision: 2.1 $
         $State: Exp $
  Last submitted $Date: 93/05/20 14:53:56 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	moseval.c,v $
 * Revision 2.1  93/05/20  14:53:56  jh
 * Fixed missing word in description.
 * 
 * Revision 2.0  93/03/30  18:04:47  jh
 * Actual first release version.
 * 
 * Revision 1.2  93/03/30  13:54:21  jh
 * General code cleanup, testing, and added features.
 * First release version.
 * 
 * Revision 1.1  92/12/15  13:08:15  jh
 * Initial revision
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/moseval.c,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef LINT
static char rcsid[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/moseval.c,v 2.1 93/05/20 14:53:56 jh Exp Locker: jh $";
static char copyright[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
/*---------------------------------------------------------------
moseval.c = quality evaluation functions for mosaic overlaps

External object files or libraries:
bilin_interp.c
-lm
---------------------------------------------------------------*/
/* inclusions */

#include <stdio.h>
#include <stdlib.h>
#include "moseval.h"
#include "bilin_interp.h"
#include "cor.h"

/*------------------------moseval--------------------------------
Effect: Evaluates the quality of a set of mosaic offsets p on the
        mosaic represented by the parameters mos, and returns
	that value.

Requires: p: array of double-precision mosaic offset values.
             The lower left frame is fixed.  p[0] is the x offset
	     of the second frame of the bottom row, p[1] the y
	     offset of that frame, p[2] the x offset of the third
	     frame, and so on to the end of the row.  Frame order
	     is from left to right, bottom to top.

	  mos: pointer to Mosevalpars for this mosaic.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
double
moseval(
	double       p[],
	Mosevalpars *mos
){
    double qual = 0.;
    size_t i;   /* indices for mos->?cor */
    size_t j;
    size_t ni = mos->ni;   /* limits of i, j */
    size_t nj = mos->nj;
    double axoff; /* offsets of frames a & b relative to frame 0,0 */
    double ayoff;
    double bxoff;
    double byoff;
    double xoff;  /* offsets of frame b relative to a */
    double yoff;
    double corx;  /* coordinate in cor image */
    double cory;

    /* loop over vertical overlaps */

    for ( i = 0 ; i < ni ; i++){
	for ( j = 0 ; j < nj - 1 ; j++){

	    /* generate offset in cor image */

	    /* look up positions in p[]; image 0,0 is special */
	    axoff = ( Mos_Xoff(i, j, nj) < 0 ? 0. : p[Mos_Xoff(i, j, nj)] );
	    ayoff = ( Mos_Yoff(i, j, nj) < 1 ? 0. : p[Mos_Yoff(i, j, nj)] );
	    bxoff = p[Mos_Xoff(i, j+1, nj)];
	    byoff = p[Mos_Yoff(i, j+1, nj)];

	    /* offset of b relative to a */
	    xoff = bxoff - axoff;
	    yoff = byoff - ayoff;

	    /* offset in cor image */
	    corx = xoff + mos->nx[i][j+1] - 1.;
	    cory = yoff + mos->ny[i][j+1] - 1.;

	    /* interpolate cor value at that offset and add to qual */
	    qual += (*mos->interp)(mos->vcor[i][j], cory, corx);

	    /* check if corx or cory were out of bounds */
	    if ( qual >= DBL_MAX ){
		return qual;
	    }
	}
    }

    /* then over horizontal ones */

    for ( i = 0 ; i < ni - 1 ; i++){
	for ( j = 0 ; j < nj ; j++){

	    /* generate offset in cor image */

	    /* look up positions in p[], image 0,0 is special */
	    axoff = ( Mos_Xoff(i, j, nj) < 0 ? 0. : p[Mos_Xoff(i, j, nj)] );
	    ayoff = ( Mos_Yoff(i, j, nj) < 1 ? 0. : p[Mos_Yoff(i, j, nj)] );
	    bxoff = p[Mos_Xoff(i+1, j, nj)];
	    byoff = p[Mos_Yoff(i+1, j, nj)];

	    /* offset of b relative to a */
	    xoff = bxoff - axoff;
	    yoff = byoff - ayoff;

	    /* offset in cor image */
	    corx = xoff + mos->nx[i+1][j] - 1.;
	    cory = yoff + mos->ny[i+1][j] - 1.;

	    /* interpolate cor value at that offset and add to qual */
	    qual += (*mos->interp)(mos->hcor[i][j], cory, corx);

	    /* check if corx or cory were out of bounds */
	    if ( qual >= DBL_MAX ){
		return qual;
	    }
	}
    }

    return qual;
}

/*------------------------create_mosevalpars---------------------
Effect: Allocates and initializes a Mosevalpars, except for the
        cor images themselves.  Returns a pointer to the
	Mosevalpars or NULL if an error occurs.

Requires: ni, nj: number of frames vertically and horizontally,
          respectively.

Modifies: nothing.

Implementation notes:
---------------------------------------------------------------*/
Mosevalpars *
create_mosevalpars(
		   size_t ni,
		   size_t nj
){
    Mosevalpars *mos;
    
    mos = (Mosevalpars *)malloc(sizeof(*mos));
    if (!mos){
	fprintf(stderr, "create_mosevalpars: unable to allocate %u bytes\n",
		sizeof(*mos));
	return NULL;
    }

    mos->ni = ni;
    mos->nj = nj;

    mos->ny = alloc_arrsize_t2(ni, nj);
    mos->nx = alloc_arrsize_t2(ni, nj);
    mos->vcor = alloc_arrdoub2p2(ni,     nj - 1);
    mos->hcor = alloc_arrdoub2p2(ni - 1, nj);
    if (!mos->nx || !mos->ny || !mos->vcor || !mos->hcor) {
	if (mos->nx){
	    free_arrsize_t2(mos->nx);
	}
	if (mos->ny){
	    free_arrsize_t2(mos->ny);
	}
	if (mos->vcor){
	    free_arrdoub2p2(mos->vcor);
	}
	if (mos->hcor){
	    free_arrdoub2p2(mos->hcor);
	}
	return NULL;
    }

    /* default functions to use */

    mos->interp = bilin_interp;
    mos->cor    = snscor;
    mos->eval   = moseval;

    return mos;
}

/*------------------------free_mosevalpars-----------------------
Effect: Frees a Mosevalpars, including the cor images.

Requires: mos: pointer to Mosevalpars to free.

Modifies: *mos.

Implementation notes:
---------------------------------------------------------------*/
void
free_mosevalpars(Mosevalpars *mos){
    free_arrdoub2p2elts(mos->hcor, mos->ni - 1, mos->nj);
    free_arrdoub2p2(mos->hcor);

    free_arrdoub2p2elts(mos->vcor, mos->ni,     mos->nj - 1);
    free_arrdoub2p2(mos->vcor);

    free_arrsize_t2(mos->nx);
    free_arrsize_t2(mos->ny);

    free(mos);

    return;
}

/*----------------------moseval_init_simplex_point---------------
Effect: Initialize base simplex point according to nominal
        overlap and regular image sizes.

Requires: mos: pointer to Mosevalpars for this mosaic.
          simp: pointer to Simplex_Param to initialize.
	  xovlap: double, nominal x overlap between frames.
	  yovlap: double, nominal y overlap between frames.
	  nx:     number of pixels in a row in all the images.
	  ny:     number of rows of pixels in all the images.

Modifies: *simp

Implementation notes: Only works if all the images are the same
                      size and have the same nominal offset.
---------------------------------------------------------------*/
void
moseval_init_simplex_point(
			   Mosevalpars   *mos,
			   Simplex_param *simp,
			   double         xovlap,
			   double         yovlap,
			   size_t         nx,
			   size_t         ny
){
    size_t         i;
    size_t         j;

    for ( i = 0 ; i < mos->ni ; i++ ){
	/* set vertical offset of all rows but the first */
	if ( i == 1 ){

	    simp->p[0][Mos_Xoff(1, 0, mos->nj)] = 0.;
	    simp->p[0][Mos_Yoff(1, 0, mos->nj)] = ny - yovlap;

	}else if ( i != 0 ){

	    simp->p[0][Mos_Xoff(i,     0, mos->nj)] = 
	    simp->p[0][Mos_Xoff((i-1), 0, mos->nj)];

	    simp->p[0][Mos_Yoff(i,     0, mos->nj)] = 
	    simp->p[0][Mos_Yoff((i-1), 0, mos->nj)] + ny - yovlap;
	}
	/* set horizontal offset relative to first item in row */
	for ( j = 1 ; j < mos->nj ; j++ ){
	    if ( i == 0 && j == 1 ){

		/* first frame in zeroth row */
		simp->p[0][Mos_Xoff(0, 1, mos->nj)] = nx - xovlap;
		simp->p[0][Mos_Yoff(0, 1, mos->nj)] = 0.;

	    } else {

		/* all other frames */
		simp->p[0][Mos_Xoff(i, j,     mos->nj)] =
		simp->p[0][Mos_Xoff(i, (j-1), mos->nj)] + nx - xovlap;

		simp->p[0][Mos_Yoff(i, j,     mos->nj)] =
		simp->p[0][Mos_Yoff(i, (j-1), mos->nj)];
	    }
	}
    }
    return;
}

/*----------------------moseval_do_cors--------------------------
Effect: Does the correlations and puts original image sizes into
        mos->nx and mos->ny.

Requires: mos: pointer to Mosevalpars for this mosaic.

          images: pointer to Doub2p2 containing all the images.

	  xovlap, yovlap: doubles, nominal number of pixels of
			  overlap between images horizontally
			  and vertically.

	  radius: double, distance from nominal overlap to which
		  to calculate correlation.

Modifies: *mos.

Implementation notes:
---------------------------------------------------------------*/
int
moseval_do_cors(
		Mosevalpars *mos,
		Doub2p2     *images,
		double       xovlap,
		double       yovlap,
		double       radius
){
    size_t i;
    size_t j;

    /* put image sizes into mos */
    for ( i = 0 ; i < mos->ni ; i++){
	for ( j = 0 ; j < mos->nj ; j++){
	    mos->ny[i][j] = images->data[i][j]->ni;
	    mos->nx[i][j] = images->data[i][j]->nj;
	}
    }

    /* loop over vertical overlaps */
    for ( i = 0 ; i < mos->ni ; i++){
	for ( j = 0 ; j < mos->nj - 1 ; j++){
	    mos->vcor[i][j] =
		(*mos->cor)(images->data[i][j], images->data[i][j+1],
			    images->data[i][j]->nj - xovlap, 0., radius);
	    if (!mos->vcor[i][j]){
		free_arrdoub2p2elts(mos->vcor, mos->ni, mos->nj - 1);
		return FAILURE;
	    }
	}
    }
    /* then over horizontal ones */
    for ( i = 0 ; i < mos->ni - 1 ; i++){
	for ( j = 0 ; j < mos->nj ; j++){
	    mos->hcor[i][j] =
		(*mos->cor)(images->data[i][j], images->data[i+1][j],
			    0., images->data[i][j]->ni - yovlap, radius);
	    if (!mos->hcor[i][j]){
		free_arrdoub2p2elts(mos->vcor, mos->ni    , mos->nj - 1);
		free_arrdoub2p2elts(mos->hcor, mos->ni - 1, mos->nj);
		return FAILURE;
	    }
	}
    }
    return SUCCESS;
}
