/*---------------------------------------------------------------
           FILE: moseval.h
      $Revision: 2.0 $
         $State: Exp $
  Last submitted $Date: 93/03/30 18:06:25 $ $Author: jh $

   Locked out by $Locker: jh $

  REVISION HISTORY
  $Log:	moseval.h,v $
 * Revision 2.0  93/03/30  18:06:25  jh
 * Actual first release version.
 * 
 * Revision 1.2  93/03/30  13:55:46  jh
 * General code cleanup, testing, and added features.
 * First release version.
 * 
 * Revision 1.1  92/12/15  13:12:43  jh
 * Initial revision
 * 

  $Source: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/moseval.h,v $

  Copyright (c) 1992, 1993 Joseph Harrington.
  All Rights reserved.
---------------------------------------------------------------*/
#ifndef MOSEVAL_PROTECT
#ifndef LINT
static char rcsidmosevalh[] = "$Header: /u3/jh/jupobs92/anal/iraf/src/jiggle/RCS/moseval.h,v 2.0 93/03/30 18:06:25 jh Exp Locker: jh $";
static char copyrightmosevalh[] =
    "Copyright (c) 1992, 1993 Joseph Harrington.  All Rights reserved.";
#endif
#endif

/*---------------------------------------------------------------
moseval.h = include file for moseval.c
---------------------------------------------------------------*/
#ifndef MOSEVAL_PROTECT
#define MOSEVAL_PROTECT

/* definitions */

#define SUCCESS 1
#define FAILURE 0
/* for p[] indices */
#define Mos_Xoff(i, j, nj) ( ((int)(i) * (int)(nj) + (int)(j) - 1) * 2 )
#define Mos_Yoff(i, j, nj) ( ((int)(i) * (int)(nj) + (int)(j) - 1) * 2 + 1 )

/* inclusions */

#include <stddef.h>
#include <float.h>
#include "memory.h"
#include "simplex.h"

/* data types */

struct st_mosevalpars {
    size_t    ni;        /* number of rows of frames in a mosaic */
    size_t    nj;        /* number of frames in a mosaic row */
    size_t  **ny;        /* number of rows of pixels in image i,j */
    size_t  **nx;        /* number of pixels in an image row in image i,j*/
    Doub2  ***vcor;      /* vertical correlations:
		                vcor[i][j] is an image of correlation values
				for the jth vertical overlap in row i */
    Doub2  ***hcor;      /* horizontal correlations (similar to vcor) */

    double  (*interp)(
		      Doub2  *z,
		      double  y,
		      double  x
	    ); /* interpolation function */

    Doub2 * (*cor)(
		   Doub2  *a, 
		   Doub2  *b,
		   double  nomx,
		   double  nomy,
		   double  radius
	    ); /* correlation function */

    double  (*eval)(
		   double                 p[],
		   struct st_mosevalpars *mos
	    ); /* mosaic evaluation function */

};
typedef struct st_mosevalpars Mosevalpars;

/* prototypes */

#if defined(__STDC__) || defined(__cplusplus)
# define P_(s) s
#else
# define P_(s) ()
#endif


/* moseval.c */
double moseval P_((double p[], Mosevalpars *pars));
Mosevalpars *create_mosevalpars P_((size_t ni, size_t nj));
void free_mosevalpars P_((Mosevalpars *pars));
void moseval_init_simplex_point P_((Mosevalpars *mos, Simplex_param *simp, double xovlap, double yovlap, size_t nx, size_t ny));
int moseval_do_cors P_((Mosevalpars *mos, Doub2p2 *images, double xovlap, double yovlap, double radius));

#undef P_

#endif
